package edu.fsu.cs.BEAT.service;

import java.util.Calendar;

import edu.fsu.cs.BEAT.backgroundtasks.UpdateRecentDataTask;
import edu.fsu.cs.BEAT.db.CategoryDbTable;
import edu.fsu.cs.BEAT.db.EntryDbTable;
import edu.fsu.cs.BEAT.db.EvenTrendDbAdapter.SqlAdapter.DatabaseHelper;
import edu.fsu.cs.BEAT.primitives.TimeSeriesCollector;
import edu.fsu.cs.BEAT.util.DateUtil;
import edu.fsu.cs.BEAT.util.Number;
import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;

public class BEATService extends Service {
	private static final String TAG = "BEATService";
	private static int lastlevel = 100;
	private static int lastbeat = 0;
	private static int lastheart = 0;

	// Input
	private TimeSeriesCollector mTSC;
	private int mHistory;
	private int mDecimals;

	// Output
	public long mLastAddId;
	public float mLastAddValue;
	public float mLastAddOldValue;
	public long mLastAddTimestamp;
	public boolean mLastAddUpdate;

	protected static final String DATABASE_NAME = "data";
	protected static final int DATABASE_VERSION = 3;

	private DatabaseHelper mDbHelper;
	private SQLiteDatabase mDb;
	private Context mCtx;
	private Calendar mCal;

	@Override
	public IBinder onBind(Intent intent) {
		return null;
	}

	@Override
	public void onCreate() {
		super.onCreate();
		// IntentFilter batteryFilter = new IntentFilter();
		// batteryFilter.addAction(android.content.Intent.ACTION_BATTERY_CHANGED);
		// registerReceiver(mIntentReceiver, batteryFilter);
		registerReceiver(mIntentReceiver,
				new IntentFilter("beat.action.update"));
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		unregisterReceiver(mIntentReceiver);
	}

	BroadcastReceiver mIntentReceiver = new BroadcastReceiver() {
		public void onReceive(Context context, Intent intent) {
			// if
			// (intent.getAction().equals(android.content.Intent.ACTION_BATTERY_CHANGED))
			// {
			// Log.d(TAG, "ACTION_BATTERY_CHANGED");
			// Bundle extras = intent.getExtras();
			// Log.d(TAG, "keys: " + extras.keySet());
			// ContentValues values = new ContentValues();
			// Long now = Long.valueOf(System.currentTimeMillis());
			// values.put("sampletime", now);
			// values.put("level", extras.getInt("level"));
			// values.put("voltage", extras.getInt("voltage"));
			// values.put("temperature", extras.getInt("temperature"));
			// values.put("plugged", extras.getInt("plugged"));
			// insertBEAT(values);
			// // if we're not charging
			// lastlevel = extras.getInt("level");
			// }

			Log.d(TAG, "beat.action.update received");
			Bundle extras = intent.getExtras();
			Log.d(TAG, "keys: " + extras.keySet());
			ContentValues values = new ContentValues();
			Long now = Long.valueOf(System.currentTimeMillis());
			values.put("sampletime", now);

			values.put("beat", extras.getString("beat"));
			values.put("heart", extras.getString("heart"));
			values.put("speed", extras.getString("speed"));
			values.put("strides", extras.getString("strides"));
			values.put("distance", extras.getString("distance"));
			values.put("cadence", extras.getString("cadence"));
			values.put("battery", extras.getString("battery"));
			//insertBEAT(values);
			//AddEntry(Float.valueOf(extras.getString("heart")).floatValue());
			// lastbeat = extras.getInt("beat");
			// lastheart = extras.getInt("heart");
		}
	};

	public void AddEntry(float value) {
		Log.d(TAG, "insertBEAT: " + value);

		CategoryDbTable.Row category = null;
		Cursor c = mDb.query(true, CategoryDbTable.TABLE_NAME,
				CategoryDbTable.KEY_ALL, CategoryDbTable.KEY_CATEGORY_NAME
						+ "=?", new String[] { "HeartRate" }, null, null, null,
				null);
		c.moveToFirst();
		if (c.getCount() > 0) {
			category = new CategoryDbTable.Row();
			category.populateFromCursor(c);
		}
		c.close();

		EntryDbTable.Row entry;
		Calendar entryTScal = Calendar.getInstance();
		Calendar lastTScal;
		long periodInMs = category.getPeriodMs();
		long timestamp = System.currentTimeMillis();
		//float value = 0.0f;

		// public long fetchCategoryId(String category)
		// fetchCategoryId("HeartRate");

		// float value =
		// Float.valueOf(mComboBox.getText().toString()).floatValue();
		entryTScal.setTimeInMillis(timestamp);

		if (periodInMs > 0) {
			UpdateRecentDataTask updater = new UpdateRecentDataTask(mTSC,
					mHistory);
			updater.setZerofill(true);
			updater.setUpdateTrend(false);
			updater.fillCategory(category.getId());
		}

		entry = mTSC.getDbh().fetchCategoryEntryInPeriod(category.getId(),
				category.getPeriodMs(), timestamp);
		if (entry == null || periodInMs == 0) {
			value = Number.Round(value, mDecimals);
			entry = new EntryDbTable.Row();
			entry.setTimestamp(timestamp);
			entry.setValue(value);
			entry.setCategoryId(category.getId());
			entry.setNEntries(1);

			mLastAddId = mTSC.getDbh().createEntry(entry);
			mLastAddValue = value;
			mLastAddOldValue = 0.0f;
			mLastAddTimestamp = timestamp;
			mLastAddUpdate = false;

		} else {
			lastTScal = Calendar.getInstance();
			lastTScal.setTimeInMillis(entry.getTimestamp());

			if ((periodInMs == DateUtil.YEAR_MS && DateUtil.inSameYear(
					entryTScal, lastTScal))
					|| (periodInMs == DateUtil.QUARTER_MS && DateUtil
							.inSameQuarter(entryTScal, lastTScal))
					|| (periodInMs == DateUtil.MONTH_MS && DateUtil
							.inSameMonth(entryTScal, lastTScal))
					|| (periodInMs == DateUtil.WEEK_MS && DateUtil.inSameWeek(
							entryTScal, lastTScal))
					|| (periodInMs == DateUtil.DAY_MS && DateUtil.inSameDay(
							entryTScal, lastTScal))
					|| (periodInMs == DateUtil.AMPM_MS && DateUtil.inSameAMPM(
							entryTScal, lastTScal))
					|| (periodInMs == DateUtil.HOUR_MS && DateUtil.inSameHour(
							entryTScal, lastTScal))) {
				if (category.getType().equals(CategoryDbTable.KEY_TYPE_SUM))
					value += entry.getValue();
				else if (category.getType().equals(
						CategoryDbTable.KEY_TYPE_AVERAGE))
					value = ((entry.getValue() * entry.getNEntries()) + value)
							/ (entry.getNEntries() + 1);
			}

			value = Number.Round(value, mDecimals);

			// entry.setTimestamp(mTimestamp);
			float oldValue = entry.getValue();
			entry.setValue(value);
			entry.setNEntries(entry.getNEntries() + 1);

			mLastAddId = entry.getId();
			mLastAddValue = value;
			mLastAddOldValue = oldValue;
			mLastAddTimestamp = timestamp;
			mLastAddUpdate = true;

			mTSC.getDbh().updateEntry(entry);
		}

		mTSC.updateCategoryTrend(category.getId());
	}

	public boolean insertBEAT(ContentValues values) {
		Log.d(TAG, "insertBEAT: " + values);

		CategoryDbTable.Row row = null;
		Cursor c = mDb.query(true, CategoryDbTable.TABLE_NAME,
				CategoryDbTable.KEY_ALL, CategoryDbTable.KEY_CATEGORY_NAME
						+ "=?", new String[] { "heart" }, null, null, null,
				null);
		c.moveToFirst();
		if (c.getCount() > 0) {
			row = new CategoryDbTable.Row();
			row.populateFromCursor(c);
		}
		c.close();

		// DatabaseHelper mOpenHelper = null;
		// SQLiteDatabase db = null;
		boolean rc = true;
		//
		// try {
		// mOpenHelper = new DatabaseHelper(this);
		// db = mOpenHelper.getReadableDatabase();
		// Long rowid = db.insert(DatabaseHelper.LEVELS_TABLE, "BEAT", values);
		// if (rowid < 0) {
		// Log.e(TAG, "database insert failed: " + rowid);
		// rc = false;
		// } else {
		// Log.d(TAG, "sample collected, rowid=" + rowid);
		// }
		// } catch (Exception e) {
		// Log.e(TAG, "database exception");
		// rc = false;
		// }
		// if (db != null) db.close();
		// if (mOpenHelper != null) mOpenHelper.close();
		return rc;
	}
}
