package edu.fsu.cs.BEATconnect.device;
/*
import zephyr.framework.api.ApiFactory;
import zephyr.framework.api.FrameworkConstants;
import zephyr.framework.bluetooth.serial.Port;
import zephyr.framework.bluetooth.serial.SearchSPP;
import zephyr.framework.bluetooth.serial.SerialPortProfile;
import zephyr.framework.command.Command;
*/

/**
 * A base class for zephyr BT device servers. Each extending class will implement parse functions. 
 * 
* <p> Created: September 27, 2008
* @author <a href="mailto:brad.zdanivsky@gmail.com">Brad Zdanivsky</a>
* 
*/
public abstract class AbstractDeviceServer implements ZephyrDevice {

	/** shared packet buffer for any type of packet */  
	public final static int BUFFER_SIZE = 256;
	public final byte[] buffer = new byte[BUFFER_SIZE];
	protected byte[] packet = null;
	
	/** BlueToothn packets will be parsed and put into a XML Command Object */ 
	Command command = null; 

	/** The smallest packet size to try to parse, used in reading/looping, see "LifeSign Packet" */
	public final static int MIN_AVAIL = 5;

	/** framework configuration */
//	protected FrameworkConstants constants = FrameworkConstants.getReference();
//	protected ApiFactory apiFactory = ApiFactory.getReference();

	/** track message inputs times */
	protected long start = System.currentTimeMillis();
	protected long last = System.currentTimeMillis();
	
	/** Serial Port Wrapper Object */ 
//	protected Port spp = null;
	protected boolean connected = false;

	/** BlueTooth device name and packet values to be parsed out */
	protected String deviceName = null;

	/**
	 * Base class for Zephyr Devices 
	 * 
	 * @param name is the blue tooth friendly name of the target device 
	 */
	public AbstractDeviceServer(String addr, String name) {

		deviceName = name;
		
		/** re-use one command object as data is just updated, overwritten while looping in device reads */  
		command = new Command(deviceName);

		/** open BT serial port */ 
//		spp = (Port) new SerialPortProfile(addr);
	}

	/**
	 * Base class for Zephyr Devices 
	 * 
	 * @param name is the blue tooth friendly name of the target device 
	 */
	public AbstractDeviceServer(String name) {

		deviceName = name;
		
		/** re-use one command object as data is just updated, overwritten while looping in device reads */  
		command = new Command(deviceName);

		/** open BT serial port */ 
//		spp = new SearchSPP(deviceName);
	}
	
	/** specific operations for each device to be overridden here */
	public abstract void readDevice();
	
	/** get the BlueTooth Name */ 
	public String getDeviceName() {
		return deviceName;
	}
	
	public String toString(){
//		return spp.toString();
		return deviceName;
	}
	
	/** Return the time in MS since the last message from the device */
	public long getElapsedTime() {
		return (System.currentTimeMillis() - start);
	}
	
	/**
	 * Used by watch dog thread 
	 * 
	 * @return the amount of time passed since last message 
	 */
	public long getDelta() {
		return (System.currentTimeMillis() - last);
	}

	/**
	 * TODO: use a 'ping' command to see if launching an new one of 'me'? 
	 */
	public void sendStatus(String string) {
		Command status = new Command("status");
		status.add("status", string);
		status.add("class", "server"); 
		status.add("deviceName", deviceName);
		status.add("delta", String.valueOf(getDelta()));
		status.add("elapsed", String.valueOf(getElapsedTime()));
		status.send();
	}
	
	/**
	 * Try to search, and connect to the device 
	 * 
	 * @return true if successfully connected, else false 
	*/
	public boolean connect() {
		
//		if( spp == null ){
//			constants.error("spp was null on connect(), " + deviceName, this); 
//			return false;
//		}
//		
//		if (spp.connect()) {
//
//			/** send notice to listening API's */ 
//			sendStatus("connected");
//			connected = true;
//
//		} else {
//			
//			/** better luck next time */ 
//			connected = false;
//		}
		
		return connected;
	} 
	
	/** Shutdown the server */
	public void close() {

		/** close the port, if it was open */ 
//		spp.close();
		
		/** drop out of any loops */ 
		connected = false;
		
		/** tell some one to restart us */
		sendStatus("closed");
		// constants.shutdown();
	}

	/** */ 
	public boolean isOpen(){
		return connected;
	}
	
	/**
	 * Get the bytes waiting in the serial port stream
	 * 
	 * @return the array of bytes read from the SPP
	*/
//	protected byte[] getAvail(Port spp) {
//
//		try {
//
//			/** don't try to parse until we get whole packet */
//			if (spp.available() <= MIN_AVAIL)
//				return null;
//
//		} catch (Exception e) {
//			//constants.error("getAvail() : " + e , this);
//			return null;
//		}
//
//		byte[] avail = null;
//		int bytes = 0;
//
//		try {
//
//			bytes = spp.read(buffer);
//
//		} catch (Exception e) {
//			constants.error("getAvail(), spp.read() : " + e , this);
//			return null;
//		}
//
//		/** read error */
//		if (bytes < 1) {
//			//constants.error("getAvail() : bytes < 1", this);
//			return null;
//		}
//
//		/** create new byte array of proper size */
//		avail = new byte[bytes];
//		ZephyrUtils.copy(avail, buffer, 0, bytes);
//		return avail;
//	} 
	
}
