package edu.fsu.cs.BEATconnect.device;

/**
 * 
 * <p> A Basic server for the Zephyr BlueTooth HXM 
 * <p> Package : Created: September 20, 2008
 * 
 * <p>See the documents here: 
 * <p> http://www.zephyrtech.co.nz/support/softwaredevelopmentkit
 * <p> http://www.zephyrtech.co.nz/assets/pdfs/bluetooth_hxm_api_guide.pdf
 * 
 * 
 * @author <a href="mailto:brad.zdanivsky@gmail.com">Brad Zdanivsky</a>
 */
public class HxmServer extends AbstractDeviceServer implements ZephyrDevice {

	/**
	 * <p> Constructor for the HxM Server API
	 * 
	 * @param name is the blue tooth friendly name of the HXM 
	 */
	public HxmServer(String name) {	
		super(name); 
	}
	
	/**
	 * <p> Constructor for the HxM Server API
	 * 
	 * @param name is the blue tooth friendly name of the HXM 
	 */
	public HxmServer(String address, String name) {
		super(address, name); 
	}
	
	/** Loop on BT input */
	public void readDevice() {
		
		/** sanity test */ 
		if( ! connected ) return;
		
		while (connected) {

//			packet = getAvail(spp);

			if (packet != null) {

				/** track arrival of data packets */
				last = System.currentTimeMillis();

				if( ZephyrUtils.vaildHxmPacket(packet)) {
					
					/** add heart rate, beat count */
					command = ZephyrUtils.parseHrmPacket(packet, command);
					
					/** add speed, distance etc */
					command = ZephyrUtils.parseHxmPacket(packet, command);
			
					/** add RR info */
					command = ZephyrUtils.parseHxmRtoR(packet, command);
					
					/** dead battery -> dead connection ? */
					if( command.get("battery").equals("0") ) {
						
//						constants.error("HXM battery is dead?", this);
//						sendStatus("deadBattery");
						
						/** Better to alert of problem - THIS IS UNDER REVIEW */
						connected = false; 
						return; 
						
					} else {
					
						// optional check 
						// if( ! command.isMalformedCommand(ZephyrUtils.HxmPrototype ))
							
						command.send();
					
					}
				} 
			}
		}
	}
}
