package edu.fsu.cs.BEATconnect.device;

import java.math.BigInteger;

/*
import zephyr.framework.api.FrameworkConstants;
import zephyr.framework.bluetooth.serial.Port;
import zephyr.framework.command.Command;
*/

import android.util.Log;


public class ZephyrUtils {
	
    // Debugging
    private static final String TAG = "ZephyrUtils";
    // private static final boolean D = true;
    public final static int PACKET_SIZE = 60;
    
	/**
	 * Known types, could re-map these to smaller tags to save band with, but less readable
	 */
	public final static String cadence = "cadence";
	public final static String strides = "strides";
	public final static String heart = "heart";
	public final static String beat = "beat";
	public final static String battery = "battery";
	public final static String speed = "speed";
	public final static String distance = "distance";
	public final static String respiration = "respiration";
	public final static String temperature = "temperature";
	public final static String posture = "posture";

	/** List the required tags for the HXM XML */
	public final static String[] HxmPrototype = { beat, heart, battery, speed,
			distance, strides, cadence, "RR0", "RR1", "RR2", "RR3", "RR4",
			"RR5", "RR6", "RR7", "RR8", "RR9", "RR10", "RR11", "RR12", "RR13" };

	/** List the required tags for the HRM XML */
	public final static String[] HrmPrototype = { beat, heart, battery, "RR0",
			"RR1", "RR2", "RR3", "RR4", "RR5", "RR6", "RR7", "RR8", "RR9",
			"RR10", "RR11", "RR12", "RR13" };

	/** List the required tags for the Bioharness XML */
	public final static String[] BioharnessPrototype = { heart, respiration,
			posture, temperature, battery,
			"RR0", "RR1", "RR2", "RR3", "RR4", "RR5", "RR6", "RR7", "RR8", "RR9",
			"RR10", "RR11", "RR12", "RR13", "RR14", "RR15", "RR16", "RR17"};

	// public final String KIND = constants.KIND;
	// public final static String RtotR = "RR"; 
	// public final static String DATA = "data"; 
	
	/** Zephyr packet constants */
	public final static int checksumPolynomial = 0x8C;

	/** HXM message id */
	public final static byte HXM_ID = 0x26;

	/** End of text */
	public final static byte ETX = 0x03;

	/** Start of text */
	public final static byte STX = 0x02;

	/** HXM packet size */
	public final static byte HXM_DLC = 0x37;

	/** Device types */
	public final static int BIOHARNESS = 1;
	public final static int HXM = 2;
	public final static int HRM = 3;

	/**
	 * Returns an instantiated Device as specified by the deviceName parameter.
	 * 
	 * @param deviceName
	 *            specifies which API to return
	 */
	public static int getDeviceType(String deviceName) {

		if (deviceName.startsWith("HXM"))
			return HXM;

		else if (deviceName.startsWith("HRM"))
			return HRM;

		else if (deviceName.startsWith("ZBH"))
			return BIOHARNESS;

		// TODO: BE RID OF OLD BIOHARNESS with space in the name!
		return BIOHARNESS;
	}


	/**
	 * Determine the type of device this is via the naming convention
	 * 
	 * @param deviceName
	 *            is the Bluetooth friendly name of the device to data log
	 * @return the ordered list of xml elements found in each type of packet
	 */
	public static String[] getPrototype(String deviceName) {

		if (getDeviceType(deviceName) == HXM)
			return HxmPrototype;

		else if (getDeviceType(deviceName) == HRM)
			return HxmPrototype;

		else if (getDeviceType(deviceName) == BIOHARNESS)
			return BioharnessPrototype;

		return BioharnessPrototype;
	}

	/**
	 * Do a CRC check on this packet
	 * 
	 * @param packet
	 *            to test
	 * @return true if error free
	 */
	public static boolean checkCRC(byte[] packet) {

		int crc = 0;
		for (int i = 2; i < 57; i++)
			crc = ZephyrUtils.ChecksumPushByte(crc, ZephyrUtils
					.readUnsignedByte(packet[i]));

		/** Then compare to the packet CRC */
		if (crc == ZephyrUtils.readUnsignedByte(packet[57]))
			return true;

		return false;
	}

	/** CRC check taken from Zephyr PDF's */
	public static int ChecksumPushByte(int currentChecksum, int newByte) {

		currentChecksum = (currentChecksum ^ newByte);

		for (int bit = 0; bit < 8; bit++) {

			if ((currentChecksum & 1) == 1)
				currentChecksum = ((currentChecksum >> 1) ^ checksumPolynomial);

			else
				currentChecksum = (currentChecksum >> 1);
		}

		return currentChecksum;
	}

	/**
	 * @param b
	 *            is the byte to convert
	 * @return a integer from the given byte
	 */
	static int readUnsignedByte(byte b) {
		return (b & 0xff);
	}

	/**
	 * Basic byte array copy
	 * 
	 * @param dest
	 * @param input
	 * @param startIndex
	 * @param size
	 */
	public static void copy(final byte[] dest, final byte[] input,
			final int startIndex, final int size) {
		int c = 0;
		for (int i = startIndex; i < size; i++)
			dest[c++] = input[i];
	}

	/**
	 * 
	 * @param packet
	 *            of bytes
	 * @param index
	 *            of the byte to parse in the byte array
	 * @return a String of the indexed byte
	 */
	public static String parseString(byte[] packet, int index) {
		String hex = byteToHex(packet[index]);
		short value = Short.parseShort(hex, 16);
		return String.valueOf(value);
	}

	/**
	 * 
	 * @param packet
	 *            of bytes
	 * @param index
	 *            of the byte to parse in the byte array
	 * @return a String of the indexed byte
	 */
	public static short parseShort(byte[] packet, int index) {
		String hex = byteToHex(packet[index]);
		return Short.parseShort(hex, 16);
	}

	/**
	 * Merge two bytes into a signed 2's complement integer
	 * 
	 * @param low
	 *            byte is LSB
	 * @param high
	 *            byte is the MSB
	 * @return a signed intt value
	 */
	public static int merge(byte low, byte high) {
		int b = 0;
		b += (high << 8) + low;
		if ((high & 0x80) != 0) {
			b = -(0xffffffff - b);
		}
		return b;
	}

	/**
	 * Merge two bytes into a unsigned integer
	 * 
	 * @param low
	 *            byte is LSB
	 * @param high
	 *            byte is the MSB
	 * @return an unsigned int value
	 */
	public static int mergeUnsigned(byte low, byte high) {
		int b = 0;
		b += (high << 8) + low;
		if ((high & 0x80) != 0) {
			b = -(0xffffffff - b);
		}
		return b;
	}

	/**
	 * Convert a byte to a hex string.
	 * 
	 * @param data
	 *            the byte to convert
	 * @return String the converted byte
	 */
	public static String byteToHex(byte data) {
		StringBuffer buf = new StringBuffer();
		buf.append(toHexChar((data >>> 4) & 0x0F));
		buf.append(toHexChar(data & 0x0F));
		return buf.toString();
	}

	/**
	 * Convert a byte array to a hex string.
	 * 
	 * @param data
	 *            the byte[] to convert
	 * @return String the converted byte[]
	 */
	public static String bytesToHex(byte[] data) {
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < data.length; i++) {
			buf.append(byteToHex(data[i]));
		}
		return buf.toString();
	}

	/**
	 * Convert an int to a hex char.
	 * 
	 * @param i
	 *            is the int to convert
	 * @return char the converted char
	 */
	public static char toHexChar(int i) {
		if ((0 <= i) && (i <= 9))
			return (char) ('0' + i);
		else
			return (char) ('a' + (i - 10));
	}

	/**
	 * Display this BT packet to the console
	 * 
	 * @param packet
	 *            of raw bytes to display in hex and byte formats
	 * 
	 *            public static void print(byte[] packet) { if (packet == null)
	 *            return; int size = packet.length; for (int c = 0; c < size;
	 *            c++) System.err.println("[" + c + "] \t hex : " +
	 *            byteToHex(packet[c]) + "\t byte : " + packet[c]); }
	 */

	/**
	 * Convert a string into an array of hex bytes
	 * 
	 * @param data
	 *            string in hex format (two chars per byte)
	 * @return the pairs as an array of bytes
	 */
	public byte[] getBytes(String data) {
		if (data == null)
			return null;
		return new BigInteger(data, 16).toByteArray();
	}

//	/**
//	 * Write a string to a SPP
//	 * 
//	 * @param bytes
//	 *            to be sent to the bioharness
//	 * @return true if the packet was sent
//	 */
//	public static boolean writeSPP(byte[] bytes, Port spp) {
//
//		try {
//
//			spp.writeBytes(bytes);
//
//		} catch (Exception e) {
//			spp.close();
//			return false;
//		}
//
//		return true;
//	}
//
//	/** Send the packet to enable data packets, 1 per second */
//	public static void setupBioharness(Port spp) {
//
//		// setup hr packets
//		byte[] data = new byte[6];
//		data[0] = STX;
//		data[1] = 0x14; // msg ID
//		data[2] = 0x01; // dlc
//		data[3] = 0x01; // turn on data stream every second
//		data[4] = 0x5e; // crc
//		data[5] = ETX;
//
//		writeSPP(data, spp);
//	}
//
//	/** Send the packet to enable data packets, 1 per second */
//	public static void setupBioharnessRtoR(Port spp) {
//
//		// setup hr packets
//		byte[] data = new byte[6];
//		data[0] = STX;
//		data[1] = 0x19; // msg ID
//		data[2] = 0x01; // dlc
//		data[3] = 0x01; // turn on data stream every second
//		data[4] = 0x5e; // crc
//		data[5] = ETX;
//
//		writeSPP(data, spp);
//	}
//
//	/** Send reset command to bioharness */
//	public static void resetBioharness(Port spp) {
//
//		// setup hr packets at one per second
//		byte[] data = new byte[6];
//		data[0] = STX;
//		data[1] = 0x1F; // msg ID
//		data[2] = 0x07; // dlc
//		data[4] = 0x5e; // crc
//		data[5] = ETX;
//
//		writeSPP(data, spp);
//	}
//
	/**
	 * Convert a raw bluetooth packet to XML command object
	 * 
	 * @param packet
	 *            packet is the raw bytes from the SPP
	 * @return Command is the same command but with the Base Bioharness elements
	 *         added
	 */
	public static Command parseBioharnessPacket(byte[] packet, Command command) {

		try {

			/** add packet type to avoid confusion with RR packets */
			// command.add(constants.KIND, DATA);

			String hrBytes = byteToHex(packet[12]);
			short hrValue = Short.parseShort(hrBytes, 16);
			command.add(heart, Short.toString(hrValue));

			int v = merge(packet[24], packet[25]);
			command.add(battery, String.valueOf(((double) v / (double) 1000)));

			int p = merge(packet[18], packet[19]);
			command.add(posture, String.valueOf(((double) p / (double) 10)));

			int r = merge(packet[14], packet[15]);
			command.add(respiration, String.valueOf(Math
					.abs(((double) r / (double) 10))));

			int t = merge(packet[16], packet[17]);
			command.add(temperature, String
					.valueOf(((double) t / (double) 10)));

		} catch (Exception e) {
			// constants.error("parseBioharnessPacket() : " + e.getMessage());
			
		}

		/** add other tags before sending ? */
		return command;
	}

	/**
	 * Convert a raw bluetooth packet an XML command object
	 * 
	 * @param packet
	 *            is the raw bytes from the SPP
	 * @return Command is the same command passed in, but with the Accelerometer
	 *         elements added
	 */
	public static Command parseHxmPacket(byte[] packet, Command command) {

		try {
			/** add packet type to avoid confusion with RR packets */
			// command.add("type", "HXM");

			command.add(strides, ZephyrUtils.parseString(packet, 54));

			// turn into a string, after scale factor applied
			int d = ZephyrUtils.mergeUnsigned(packet[50], packet[51]);
			command.add(distance, String.valueOf(Math
					.abs(((double) d / (double) 16))));

			int s = ZephyrUtils.mergeUnsigned(packet[52], packet[53]);
			command.add(speed, String.valueOf(Math
					.abs(((double) s / (double) 256))));

			int c = ZephyrUtils.mergeUnsigned(packet[54], packet[55]);
			command.add(cadence, String.valueOf(Math
					.abs(((double) c / (double) 16))));

		} catch (Exception e) {
			// constants.error("parseHxmPacket() : " + e.getMessage());
            Log.e(TAG, "parseHxmPacket() : ", e);		
		}

		/** add other tags before sending ? */
		return command;
	}

	/**
	 * Convert a raw bluetooth packet an XML command object
	 * 
	 * @param packet
	 *            is the raw bytes from the SPP
	 * @return Command is the same command but with the HRM elements added
	 */
	public static Command parseHrmPacket(byte[] packet, Command command) {

		command.add(battery, ZephyrUtils.parseString(packet, 11));
		command.add(heart, ZephyrUtils.parseString(packet, 12));
		command.add(beat, ZephyrUtils.parseString(packet, 13));

		return command;
	}

	/**
	 * Convert a raw bluetooth packet an XML command object
	 * 
	 * @param packet
	 *            is the raw bytes from the SPP
	 * @return Command is the same command but with the RR elements added
	 */
	public static Command parseHxmRtoR(byte[] packet, Command command) {

		//command.add(constants.KIND, RtotR);
		
		/** first rr time stamp index, see pdf */
		int index = 14;
		for (int i = 0; i < 15; i++) {

			/** two bytes per time stamp */
			command.add("RR" + String.valueOf(i), String.valueOf(Math
					.abs(ZephyrUtils.mergeUnsigned(packet[index],
							packet[index + 1]))));

			/** each time stamp is two bytes wide */
			index += 2;
		}
		return command;
	}

	/**
	 * Convert a raw bluetooth packet an XML command object
	 * 
	 * @param packet
	 *            is the raw bytes from the SPP
	 * @return Command is the same command but with the RR elements added
	 */
	public static Command parseBioharnessRtoR(byte[] packet, Command command) {

		//command.add(beat, ZephyrUtils.parseString(packet, 3));

		int j = 0;
		for (int i = 12; i < 48; i += 2) {

			command.add("RR" + String.valueOf(j++), String.valueOf(Math.abs(merge(
					packet[i], packet[i + 1]))));

		}

		/** add other tags before sending ? */
		return command;
	}

	/**
	 * Check if this is valid HXM packet
	 * 
	 * @param packet
	 *            of raw bytes to test before parsing
	 * @return true if this is an error free transmission
	 */
	public static boolean vaildHxmPacket(byte[] packet) {

		if (packet == null)
			return false;

		if (packet.length != PACKET_SIZE) {
			/** most common, happens when not in sync with HXM */
			// constants.error("wrong packet size on HXM");
			Log.i(TAG, "wrong packet size on HXM");
			return false;
		}

		if (packet[0] != STX) {
			// constants.error("STX error on HXM");
			Log.i(TAG, "STX error on HXM");
			return false;
		}

		if (packet[1] != HXM_ID) {
			// constants.error("MSG_ID error on HXM");
			Log.i(TAG, "MSG_ID error on HXM");
			return false;
		}

		if (packet[2] != HXM_DLC) {
			// constants.error("DLC error on HXM");
			Log.i(TAG, "DLC error on HXM");
			return false;
		}

		if (packet[59] != ETX) {
			// constants.error("ETC error on HXM");
			Log.i(TAG, "ETC error on HXM");
			return false;
		}

		if (ZephyrUtils.checkCRC(packet)) {
			// constants.error("CRC error on HXM");
			Log.i(TAG, "CRC error on HXM");
			return false;
		}

		/** all is well, parse this one */
		return true;
	}

	/**
	 * Check this is valid packet
	 * 
	 * @param packet to test
	 * @return true if this is an error free transmission
	 */
	public boolean vaildBioharnessPacket(byte[] packet) {

		if (packet == null) return false;

		// if (packet == null) return ERROR;
		// if (packet.length < 5) return ERROR;

		// vaild packet ?
		// if (packet[0] != ZephyrUtils.STX) return ERROR;

		// if( packet.length != 60 ) return false;
		// if( packet[0] != ZephyrUtils.STX ) return false;
		// if( packet[1] != MSG_ID ) return false;
		// if( packet[2] != DLC ) return false;
		// if( packet[59] != ZephyrUtils.ETX ) return false;

		//
		// calculating the crc is optional, beat count is more than enough
		// protection?
		//
		/*
		 * if( ZephyrUtils.checkCRC(packet) ) {
		 * 
		 * constants.error("crc error on device [" + deviceName + "]", this);
		 * return false; }
		 */

		return true;
	}

}
